% Example_6_1.m
% Calculate ramp response of RC circuit via 3 methods:
% 1. exact solution
% 2. by Euler method
% 3. by modified Euler method
clc; clear;
% Circuit parameters:
R=1e3;      % ohms
C=1e-6;     % farads
tau = R*C;  % seconds
Vo = 10;    % volts
f = @(tm,Vc) (1/tau)*(Vo*tm-Vc); % Equation 6.17
% Parameters for Euler algorithm
interval = 10*tau;
steps = 20;
h = interval/steps;
% Create set of time points to calculate
t = 0:h:interval;
% Initial conditions
Vc_exact(1)=0;
Vc_euler(1)=0;
Vc_mod_euler(1)=0;
% Exact and Euler method: march in time
for i=1:length(t)-1
    % Apply Equation 6.16
    Vc_exact(i+1) = Vo*(t(i+1)-tau+tau*exp(-t(i+1)/tau));
    % Apply Equation 6.4
    Vc_euler(i+1) = Vc_euler(i) + h*f(t(i),Vc_euler(i));
end
% Modified Euler: apply corrector equation
tolerance = 1e-4; % .01 percent
for i=1:length(t)-1
    % use the Euler method to calculate an initial guess
    % for the predicted value
    Vc_predicted =  Vc_mod_euler(i) ...
        + h*f(t(i),Vc_mod_euler(i));
    % Repeatedly apply the corrector equation until the
    % predictor and corrector are equal (or within
    % tolerance)
    for j=1:50
        Vc_corrected = Vc_mod_euler(i) ...
            + (h/2)*(f(t(i),Vc_mod_euler(i)) ...
            + f(t(i+1),Vc_predicted));
        if abs(Vc_corrected-Vc_predicted)/Vc_predicted...
                < tolerance
            break;
        else
            Vc_predicted = Vc_corrected;
        end
    end
    % if we reached the iteration limit of 50, then print
    % warning:
    if j==50
        fprintf('Warning at step ');
        fprintf('%d: reached iteration limit\n',i);
    end
    Vc_mod_euler(i+1)=Vc_corrected;
end
% Plot results
subplot(1,2,1), plot(t,Vc_euler,'x',t,Vc_exact),
legend('Euler','Exact','Location','NorthWest');
title('Euler solution compared to exact');
xlabel('seconds'); ylabel('Vc (volts)');
text(0.006,0.005,sprintf('%d steps',steps));
subplot(1,2,2), plot(t,Vc_mod_euler,'x',t,Vc_exact),
legend('Modified Euler','Exact','Location','NorthWest');
title('Modified Euler solution compared to exact');
xlabel('seconds'), ylabel('Vc (volts)');
text(0.006,0.005,sprintf('%d steps',steps));
