% Example_8_4.m
% This example implements an FIR low-pass filter using a
% truncated sinc function (equivalent to an ideal filter
% with a rectangular window)
clc; clear;
% define sampling rate:
fs = 40000; T=1/fs;
% define cutoff frequency of low-pass filter:
fo = 1000;
% define frequencies of input signal x(t):
input_freqs = [ 500 1500 3000 ];
% define filter order (N):
taps = 401;  N = taps-1;
% Note: from this point forward, we will operate in the
% dimensionless discrete-time "n" domain.  First, we
% redefine the filter cutoff frequency in terms of the
% dimensionless Phi parameter of the DTFT:
phi_o = fo / fs;
% Define hr[n] as a shifted sinc function over 401 taps
for i=1:taps
    n = i-1;
    shifted_n = ( n - N/2 );
    hr(i) = sin(2 * pi * phi_o * shifted_n) ...
        / (pi * shifted_n);
    % Note: since sin(x)/x is singular for x=0, catch
    % this case and insert the appropriate limit value.
    % Otherwise, our hr[n] won't work because it will
    % have a single "NaN" value at the peak.
    if shifted_n==0
        hr(i) = 2*phi_o;
    end
end
% Run the filter for a long time so that we can be sure
% to reach the steady state.  Arbitrarily choose 20000
% samples.
max_n = fs/2;
n = 0:max_n;
% Define x[n]. First, rescale the input frequencies to be
% fractions of fs:
input_phis = input_freqs / fs;
x = zeros(1,length(n));
for i=1:length(input_phis)
    x = x + sin(2*pi*input_phis(i)*n);
end
% Shift x[n] to "turn on" at a time greater than the
% number of taps of the filter.  This is equivalent to
% setting the initial conditions of the filter to zero.
x_shifted = zeros(1,length(n));
x_shifted(taps:length(n)) = x(1:length(n)-N);
% Compute y[n] by simulating the delay circuit of Figure
% 8.12. This is equivalent to convolving x[n] with hr[n].
y = zeros(1,length(n));
for i=taps:length(n)
    for j=1:taps
        % calculate the value for each tap and add to the
        % running total
        y(i) = y(i) + hr(j)*x_shifted(i-j+1);
    end
end
% Convert the discrete signals hr[n], x[n], and y[n] to
% the time domain by rescaling the horizontal axis and
% connecting the dots.
interesting_interval = 1400*T;
subplot(3,1,1);
plot(T*n(1:taps), hr);
% rescale the axes to see the interesting parts:
axis([0 interesting_interval -.02 .06]);
ylabel('h_R(t)');
title('FIR filter w/ rectangular window, time domain');
subplot(3,1,2);
plot(T*n,x_shifted);
axis([0 interesting_interval -3 3]);
ylabel('x(t)');
subplot(3,1,3);
plot(T*n,y);
ylabel('y(t)'); xlabel('t (sec)');
axis([0 interesting_interval -1.5 1.5]);
figure;
% See how good our filter is by computing a 4096-point
% DFT of x[n] and y[n] and plotting on semilog axes.
% Rescale the horizontal axis to be in units of frequency
% instead of the dimensionless "k" parameter of the DFT.
fft_points = 4096;
scaled_freqs = 0:fs/(fft_points-1):fs;
interesting_freq = 6000;
subplot(3,1,1);
HR = fft(hr,fft_points);
semilogy(scaled_freqs,abs(HR));
ylabel('|H_R(f)|');
% rescale the axes to see the interesting parts:
axis([0 interesting_freq 1e-4 1e1]);
title('FIR filter w/ rectangular window, freq domain');
subplot(3,1,2);
% Calculate the fft on the end part of x[n] and y[n] so
% that we are sure that we are in the steady state
% portion.
x_end = x(end-fft_points+1:end);
X=fft(x_end);
semilogy(scaled_freqs,abs(X));
axis([0 interesting_freq 1e-1 1e4 ]);
ylabel('|X(f)|');
subplot(3,1,3);
y_end = y(end-fft_points+1:end);
Y=fft(y_end);
semilogy(scaled_freqs,abs(Y));
axis([0 interesting_freq 1e-1 1e4 ]);
ylabel('|Y(f)|');
xlabel('f (Hz)');
